
/*
 *  CPlan solves planning problems formulated as constraint satisfaction
 *  problems.
 *
 *  Copyright (C) 1999  Peter van Beek and Xinguang Chen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.  See the GNU General Public License
 *  for more details (see the file called Copying or contact the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA).
 */

/*
 *  Variable/value types for grid planning problems.
 */
#define  LOCK      0    /* variable type                        */
#define  KEY       1    /* variable type                        */
#define  ROBOT     2    /* variable type                        */
#define  ARM       3    /* variable type                        */
#define  GRID      4    /* value type (for robot and key)       */
#define  HELD      5    /* value type (for keys)                */
#define  EMPTY     6    /* value type (for arm)                 */
#define  LOCKED    7    /* value type (for locks)               */
#define  UNLOCKED  8    /* value type (for locks)               */

/*
 *  Types of constraints for state-based approach.
 */

#define  KEY_DISTANCE_CONSTRAINT        0
#define  ROBOT_DISTANCE_CONSTRAINT      1
#define  ACTION_MOVE_CONSTRAINT         2
#define  ACTION_PICKUP_CONSTRAINT       3
#define  ACTION_PICKUP_LOOSE_CONSTRAINT 4
#define  ACTION_PUTDOWN_CONSTRAINT      5
#define  ACTION_UNLOCK_CONSTRAINT       6
#define  USELESS_ACTION_CONSTRAINT      7
#define  STATE_CONSTRAINT               8
#define  DETENT_CONSTRAINT              9
#define  NOENTRY_CONSTRAINT             10
#define  KEY_DISTANCE2_CONSTRAINT       11
#define  ROBOT_DISTANCE2_CONSTRAINT     12
#define  KEY_DISTANCE3_CONSTRAINT       13
#define  ROBOT_DISTANCE3_CONSTRAINT     14
#define  LAST                           15

/*
 *  Struct for an instance of the grippers problems.
 */
typedef struct {
    int  problem;
    int  size;
    int  locks;
    int  keys;
    int  robot_orig;
    int  *lock_orig;
    int  *lock_shape;
    int  *key_orig;
    int  *key_shape;
    int  *key_dest;
    int  *key_name;
    int  **grid_table;
} I_type;


void	get_instance();
void	print_instance();
void	dump_instance();
int	get_lower_bound();
int     get_upper_bound();

int	get_size();
int	get_num_locks();
int	get_num_keys();

int	get_robot_orig();

int	get_key_orig();
int	get_key_shape();
int	get_key_dest();
int	get_key_name();

int	get_lock_orig();
int	get_lock_shape();

int	grid_distance();

