
/*
 *  CPlan solves planning problems formulated as constraint satisfaction
 *  problems.
 *
 *  Copyright (C) 1999  Peter van Beek and Xinguang Chen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.  See the GNU General Public License
 *  for more details (see the file called Copying or contact the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA).
 */

#include "grid.h"
#include "../Share/csp.h"
#include "../Share/global.h"

void
process_solution( C, solution )
    CSP_type    *C;
    int         *solution;
{
    int         t, i;
    int         v1, v2;
    VALA        a1, a2;

    for( t = 0; t <= C->time-1; t++ ) {
        printf( "Action at time %2d:\n", t+1 );
        /*
         * LOCK variables
         */
        for( i = 1; i <= get_num_locks(); i++ ) {
            v1 = get_var( C, t,   LOCK, i );
            v2 = get_var( C, t+1, LOCK, i );
            a1 = C->value_attribute[v1][solution[v1]];
            a2 = C->value_attribute[v2][solution[v2]];

            if( a1.type == LOCKED &&
                a2.type == UNLOCKED ) {
                printf( "\tnode%d-%d is unlocked\n",
			a1.number/get_size(),
			a1.number%get_size() );
            }
        }
        /*
         * KEY variables
         */
        for( i = 1; i <= get_num_keys(); i++ ) {
            v1 = get_var( C, t,   KEY, i );
            v2 = get_var( C, t+1, KEY, i );
            a1 = C->value_attribute[v1][solution[v1]];
            a2 = C->value_attribute[v2][solution[v2]];

            if( a1.type == GRID &&
                a2.type == HELD ) {
                printf( "\tpick up key%d at node%d-%d\n",
			get_key_name( i ),
			a1.number/get_size(),
			a1.number%get_size() );
            }
            else
            if( a1.type == HELD &&
                a2.type == GRID ) {
                printf( "\tputdown key%d at node%d-%d\n",
			get_key_name( i ),
			a2.number/get_size(),
			a2.number%get_size() );
            }
        }
        /*
         * ROBOT variable
         */
        v1 = get_var( C, t,   ROBOT, 1);
        v2 = get_var( C, t+1, ROBOT, 1);
        a1 = C->value_attribute[v1][solution[v1]];
        a2 = C->value_attribute[v2][solution[v2]];

        if ( a1.number != a2.number) {
             printf("\tmove robot from node%d-%d to node %d-%d\n",
                     a1.number/get_size(), a1.number%get_size(),
                     a2.number/get_size(), a2.number%get_size() );
        }
    }
}

