
/*
 *  CPlan solves planning problems formulated as constraint satisfaction
 *  problems.
 *
 *  Copyright (C) 1999  Peter van Beek and Xinguang Chen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.  See the GNU General Public License
 *  for more details (see the file called Copying or contact the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA).
 */

#include "gripper.h"
#include "../Share/global.h"

#define MAX_STEPS 100


static   I_type	*L;


void
get_instance( problem )
    int		problem;
{
    int		i, number, room;
    char	name[50];
    FILE	*fp;

    if( problem < 10 ) sprintf( name, "./Problems/p0%1d", problem );
    else	       sprintf( name, "./Problems/p%2d",  problem );

    if( (fp = fopen( name, "r" )) == NULL ) {
	printf( "Could not open file %s\n", name );
	exit( 0 );
    }

    L = (I_type *)new_calloc( 1, sizeof(I_type) );

    /*
     *  Get and set parameters.
     */
    L->problem = problem;
    fscanf( fp, "Parameters:\n" );
    fscanf( fp, "\tname: %s\n", name);
    fscanf( fp, "\tballs: %d\n", &(L->balls) );
    fscanf( fp, "\tgrippers: %d\n", &(L->grippers) );
    fscanf( fp, "\trooms: %d\n", &(L->rooms) );

    /*
     *  Allocate space to store the initial and goal states.
     */
    L->orig_room = new_calloc( L->balls+1, sizeof(int) );
    L->dest_room = new_calloc( L->balls+1, sizeof(int) );

    /*
     *  Get initial state.
     */
    fscanf( fp, "Initial:\n" );
    fscanf( fp, "\t(at-robby room%d)\n", &L->robby_orig_room );

    for( i = 1; i <= L->grippers; i++ ) {
	fscanf( fp, "\t(free %s)\n", name );
    }

    for( i = 1; i <= L->balls; i++ ) {
	fscanf( fp, "\t(at ball%d room%d)\n", &number, &room );
	L->orig_room[number] = room;
    }

    /*
     *  Get goal state.
     */
    fscanf( fp, "Goal:\n" );
    for( i = 1; i <= L->balls; i++ ) {
	fscanf( fp, "\t(at ball%d room%d)\n", &number, &room );
        L->dest_room[number] = room;
    }

    fclose( fp );
}

void
print_instance()
{
    printf( "problem: %2d  (%d balls, %d grippers, %d rooms)\n", 
            L->problem, L->balls, L->grippers, L->rooms );
}

void
dump_instance()
{
    int         i;

    printf( "Initial:\n" );
    printf( "\t(at-robby room%d)\n", L->robby_orig_room);
    for( i = 1; i <= L->balls; i++ ) {
        printf( "\t(at ball%d room%d)\n", i, L->orig_room[i]);
    }
    printf( "Goal:\n" );
    for( i = 1; i <= L->balls; i++ ) {
        printf( "\t(at ball%d room%d)\n", i, L->dest_room[i]);
    }
}


int get_num_balls()		{ return( L->balls ); }
int get_num_grippers()		{ return( L->grippers ); }
int get_num_rooms()		{ return( L->rooms ); }
int get_orig_room( int a )	{ return( L->orig_room[a] ); }
int get_dest_room( int a )	{ return( L->dest_room[a] ); }
int get_robby_orig_room( )	{ return( L->robby_orig_room ); }

int get_upper_bound()		{ return( MAX_STEPS ); }

int
get_lower_bound()
{
    int         i;
    int         bound1, bound2;
    int         moves[3];

    /*
     * Calculate the number of balls at each room that
     * need to be moved.
     */

    moves[1] = moves[2] = 0;
    for( i = 1; i <= L->balls; i++) {
    	if( L->orig_room[i] != L->dest_room[i] ) {
	    moves[L->orig_room[i]] ++;
	}
    }

    if( moves[1] % L->grippers ) {
        bound1 = moves[1] / L->grippers + 1;
    }
    else {
        bound1 = moves[1] / L->grippers;
    }
    if( moves[2] % L->grippers ) {
        bound2 = moves[2] / L->grippers + 1;
    }
    else {
        bound2 = moves[2] / L->grippers;
    }

    bound1 = bound1 * 4 - 1;
    bound2 = bound2 * 4 - 1;
    if( bound1 > bound2 ) {
    	return( bound1 );
    }
    else {
    	return( bound2 );
    }
}

