
/*
 *  CPlan solves planning problems formulated as constraint satisfaction
 *  problems.
 *
 *  Copyright (C) 1999  Peter van Beek and Xinguang Chen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.  See the GNU General Public License
 *  for more details (see the file called Copying or contact the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA).
 */

/*
 *  Variable/value types for gripper planning problems.
 */
#define  BALL      0    /* variable and value type (for grippers) */
#define  GRIPPER   1    /* variable and value type (for balls)    */
#define  ROBOT     2    /* variable type                          */
#define  EMPTY     3    /* value type (for grippers)              */
#define  ROOM      4    /* value type (for balls and robot)       */

/*
 *  Types of constraints for state-based approach.
 */
#define  DISTANCE_CONSTRAINT            0
#define  ACTION_PICKUP_DROP_CONSTRAINT  1
#define  SYMMETRY_CONSTRAINT            2
#define  USELESS_ACTION_CONSTRAINT      3
#define  USELESS2_ACTION_CONSTRAINT	4
#define  STATE_CONSTRAINT          	5
#define  LOWER_BOUND_CONSTRAINT		6
#define  LAST                           7

/*
 *  Struct for an instance of the grippers problems.
 */
typedef struct {
    int  problem;
    int  balls;			/* number of balls */
    int  grippers;		/* number of gripper */
    int  rooms;			/* number of rooms. fixed to 2 */
    int  robby_orig_room;	/* robot's original room */
    int  *orig_room;		/* balls' original room */
    int  *dest_room;		/* balls; distination room */
} I_type;


void	get_instance();
void	print_instance();
void	dump_instance();

int	get_num_balls();
int	get_num_grippers();
int	get_num_rooms();
int	get_orig_room();
int	get_dest_room();
int	get_robby_orig_room( );
int	get_lower_bound();
int	get_upper_bound();

