
/*
 *  CPlan solves planning problems formulated as constraint satisfaction
 *  problems.
 *
 *  Copyright (C) 1999  Peter van Beek and Xinguang Chen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.  See the GNU General Public License
 *  for more details (see the file called Copying or contact the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA).
 */

#include "gripper.h"
#include "../Share/csp.h"
#include "../Share/global.h"

void
process_solution( C, solution )
    CSP_type    *C;
    int         *solution;
{
    int         t, i;
    int         v1, v2;
    VALA        a1, a2;

    for( t = 0; t <= C->time-1; t++ ) {
        printf( "Action at time %2d:\n", t+1 );
        /*
         * Ball variables
         */
        for( i = 1; i <= get_num_balls(); i++ ) {
            v1 = get_var( C, t,   BALL, i );
            v2 = get_var( C, t+1, BALL, i );
            a1 = C->value_attribute[v1][solution[v1]];
            a2 = C->value_attribute[v2][solution[v2]];

            if( a1.type == ROOM &&
                a2.type == GRIPPER ) {
                printf( "\tpickup ball %d by gripper %d in room %d\n",
                        i, a2.number, a1.number );
            }
            else
            if( a1.type == GRIPPER &&
                a2.type == ROOM ) {
                printf( "\tputdown ball %d from gripper %d in room %d\n",
                        i, a1.number, a2.number );
            }
        }

        v1 = get_var( C, t,   ROBOT, 1);
        v2 = get_var( C, t+1, ROBOT, 1);
        a1 = C->value_attribute[v1][solution[v1]];
        a2 = C->value_attribute[v2][solution[v2]];

        if ( a1.number != a2.number) {
             printf("\tmove robot from room %d to room %d\n",
                     a1.number, a2.number);
        }
    }


}

