
/*
 *  CPlan solves planning problems formulated as constraint satisfaction
 *  problems.
 *
 *  Copyright (C) 1999  Peter van Beek and Xinguang Chen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.  See the GNU General Public License
 *  for more details (see the file called Copying or contact the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA).
 */

/*
 *  Variable/value types for logistics problems.
 */
#define  PLANE	   0	/* variable and value type */
#define  TRUCK	   1	/* variable and value type */
#define  PACKAGE   2	/* variable type	   */
#define  LOCATION  3	/* value type		   */

/*
 *  Types of constraints for logistics problems.
 */
#define  DISTANCE_CONSTRAINT                0
#define  DISTANCE2_CONSTRAINT               1
#define  SUCCESSFUL_LOAD_UNLOAD_CONSTRAINT  2
#define  USELESS_ACTIONS_CONSTRAINT         3
#define  FORCE_PARALLELISM_CONSTRAINT       4
#define  OUTGOING_CONSTRAINT                5
#define  INCOMING_CONSTRAINT                6
#define  ENROUTE_CONSTRAINT		    7
#define  UPPER_BOUND_CONSTRAINT		    8
#define  SYMMETRY_CONSTRAINT		    9
#define  SYMMETRY2_CONSTRAINT		    10
#define  SYMMETRY3_CONSTRAINT		    11
#define  SYMMETRY4_CONSTRAINT		    12
#define  FORCE_MOVE_CONSTRAINT		    13
#define  LAST				    14

/*
 *  Struct for an instance of the logistics problems.
 */
typedef struct {
    int  problem;
    int	 packages;		/* number of packages			     */
    int  planes;		/* number of planes			     */
    int  cities;		/* number of cities			     */
    int  locations_per_city;	/* number of locations in each city	     */
    int  trucks;		/* number of trucks			     */
    int  *orig_city;		/* initial city of each package		     */
    int  *orig_loca;		/* initial location in city of each package  */
    int  *dest_city;		/* destination city of each package	     */
    int  *dest_loca;		/* destination location of each package	     */
    int  *plane_orig_city;	/* initial city of each plane		     */
    int  *plane_orig_loca;	/* initial location in city of each plane    */
    int  *truck_orig_city;	/* initial city of each truck		     */
    int  *truck_orig_loca;	/* initial location in city of each truck    */
    int  *airport_loca;		/* airport location of each city	     */
    int  *outgoing;		/* number of outgoing packages for each city */
    int  *local;		/* number of local packages for each city    */
    int  *incoming;		/* number of incoming packages for each city */
    int  *trucks_in_city;	/* number of trucks for each city            */
    int  *package_name;         /* names of packages for outputing plan      */
    int  *plane_name;           /* names of planes for outputing plan        */
    int  *truck_name;           /* names of trucks for outputing plan        */
} I_type;

/*
 *  Routines available for an instance of the logistics problems.
 */
void    get_instance();
void    print_instance();
void    dump_instance();

int     get_num_packages();
int     get_num_planes();
int     get_num_cities();
int     get_loc_per_city();
int     get_num_trucks();

int     get_upper_bound();
int     get_lower_bound();
int     upper_bound_outgoing();
int     upper_bound_local();
int     upper_bound_incoming();

int	get_package_name();
int	get_plane_name();
int	get_truck_name();

int	needs_plane();			/* one package  */
int	same_origination();		/* two packages */
int	same_origination_city();	/* two packages */
int	same_destination();		/* two packages */
int	same_destination_city();	/* two packages */
int	indistinguishable_planes();	/* two planes   */
int	all_planes_indistinguishable();
int	indistinguishable_trucks();	/* two trucks   */
int	force_unload();

int	possible_transfer();
int	possible_local_transfer();

int     get_orig_city();
int     get_orig_loca();
int     get_dest_city();
int     get_dest_loca();
int     get_plane_orig_city();
int     get_plane_orig_loca();
int     get_truck_orig_city();
int     get_truck_orig_loca();
int     get_trucks_in_city();
int     get_airport_loca();

