
/*
 *  CPlan solves planning problems formulated as constraint satisfaction
 *  problems.
 *
 *  Copyright (C) 1999  Peter van Beek and Xinguang Chen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.  See the GNU General Public License
 *  for more details (see the file called Copying or contact the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA).
 */

#include "logistics.h"
#include "../Share/global.h"
#include "../Share/csp.h"

#define CC_LIMIT  50000000000.0   /* limit on consistency checks */


void
usage()
{
    printf( "Usage: test problem\n" );
    printf( "    problem  -- logistics problem [1-65]\n" );
    exit( 6 );
}


int
main( argc, argv )
    int		argc;
    char	*argv[];
{
    CSP_type    *C;
    int		problem, steps, max_steps;

    if( argc != 2 ) {
	usage();
    }

    problem = atoi( argv[1] );
    if( (problem < 1) || (problem > 65) ) {
	usage();
    }

    set_limit( CC_LIMIT );
    start_timer();

    get_instance( problem );
    print_instance();

    steps = get_lower_bound();
    max_steps = get_upper_bound();

    do {
	C = generate( steps );

	solve( C );

	free_CSP( C );

	printf( "Stats (%2d,%2d): ", problem, steps );
	printf( "checks: %10.0f, visits: %7.0f, solution: %1.0f, time: %7.2f\n",
	    checks, visits, solution_count, elapsed_time() );

	fflush( stdout );

	steps++;

    } while( solution_count == 0 && steps <= max_steps );

    exit( 0 );
}

