
/*
 *  CPlan solves planning problems formulated as constraint satisfaction
 *  problems.
 *
 *  Copyright (C) 1999  Peter van Beek and Xinguang Chen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.  See the GNU General Public License
 *  for more details (see the file called Copying or contact the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA).
 */

#include "logistics.h"
#include "../Share/global.h"
#include "../Share/csp.h"

static int
same_attributes( a, b )
    VALA	a, b;
{
    return( a.type   == b.type &&
	    a.city   == b.city &&
	    a.number == b.number );
}

void
process_solution( C, solution )
    CSP_type    *C;
    int		*solution;
{
    int		t, i, v1, v2;
    VALA	a, b;

    for( t = 0; t <= C->time-1; t++ ) {
	printf( "Actions at time %d\n", t+1 );
	/*
	 * Package variables
	 */
        for( i = 1; i <= get_num_packages(); i++ ) {
	    v1 = get_var( C, t,   PACKAGE, i );
	    v2 = get_var( C, t+1, PACKAGE, i );
	    a = C->value_attribute[v1][solution[v1]];
	    b = C->value_attribute[v2][solution[v2]];

	    if( !same_attributes( a, b ) ) {
		if( a.type == LOCATION &&
		    b.type == PLANE )
		    printf( "\tload package%d from %d.%d onto plane%d\n",
			    get_package_name( i ),
			    a.city, a.number,
			    get_plane_name( b.number ) );
		else
		if( a.type == LOCATION &&
		    b.type == TRUCK )
		    printf( "\tload package%d from %d.%d onto truck%d\n",
			    get_package_name( i ),
			    a.city, a.number,
			    get_truck_name( b.number ) );
		else
		if( a.type == PLANE &&
		    b.type == LOCATION )
		    printf( "\tunload package%d from plane%d to %d.%d\n",
			    get_package_name( i ),
			    get_plane_name( a.number ),
			    b.city, b.number );
		else
		if( a.type == TRUCK &&
		    b.type == LOCATION )
		    printf( "\tunload package%d from truck%d to %d.%d\n",
			    get_package_name( i ),
			    get_truck_name( a.number ),
			    b.city, b.number );
		else
		    printf( "error\n" );
	    }
        }
	/*
	 * Plane variables
	 */
        for( i = 1; i <= get_num_planes(); i++ ) {
	    v1 = get_var( C, t,   PLANE, i );
	    v2 = get_var( C, t+1, PLANE, i );
	    a = C->value_attribute[v1][solution[v1]];
	    b = C->value_attribute[v2][solution[v2]];

	    if( !same_attributes( a, b ) ) {
		printf( "\tfly plane%d from %d.%d to %d.%d\n",
			get_plane_name( i ),
			a.city, a.number,
			b.city, b.number );
	    }
        }
	/*
	 * Truck variables
	 */
        for( i = 1; i <= get_num_trucks(); i++ ) {
	    v1 = get_var( C, t,   TRUCK, i );
	    v2 = get_var( C, t+1, TRUCK, i );
	    a = C->value_attribute[v1][solution[v1]];
	    b = C->value_attribute[v2][solution[v2]];

	    if( !same_attributes( a, b ) ) {
		printf( "\tdrive truck%d from %d.%d to %d.%d\n",
			get_truck_name( i ),
			a.city, a.number,
			b.city, b.number );
	    }
        }
    }
}

