
/*
 *  CPlan solves planning problems formulated as constraint satisfaction
 *  problems.
 *
 *  Copyright (C) 1999  Peter van Beek and Xinguang Chen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.  See the GNU General Public License
 *  for more details (see the file called Copying or contact the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA).
 */

#include "mystery.h"
#include "../Share/csp.h"
#include "../Share/global.h"

#define CC_LIMIT  50000000000.0   /* limit on consistency checks */


void
usage()
{
    printf( "Usage: test problem [squirt]\n" );
    printf( "    problem  -- mystery problem [1-35]\n" );
    printf( "    squirt   -- squirt action is available\n" );
    exit( 6 );
}


int
main( argc, argv )
    int		argc;
    char	*argv[];
{
    CSP_type    *C;
    int		problem, steps, max_steps, squirt;

    if( argc != 2 && argc != 3 ) {
	usage();
    }

    problem = atoi( argv[1] );
    if( (problem < 1) || (problem > 35) ) {
	usage();
    }

    squirt = 0;
    if( argc == 3 ) {
	squirt = (strcmp( argv[2], "squirt" ) == 0);
    }

    set_limit( CC_LIMIT );
    start_timer();

    get_instance( problem, squirt );
    print_instance();

    steps = get_lower_bound();
    max_steps = get_upper_bound();
    solution_count = 0;

    while( steps <= max_steps && solution_count == 0 ) {

	C = generate( steps );

	solve( C );

	free_CSP( C );

	printf( "Stats (%2d,%2d): ", problem, steps );
	printf( "checks: %10.0f, visits: %7.0f, solution: %1.0f, time: %7.2f\n",
	    checks, visits, solution_count, elapsed_time() );

	fflush( stdout );

	steps++;
    }

    printf( "Total time: %7.2f\n", elapsed_time() );

    exit( 0 );
}

