
/*
 *  CPlan solves planning problems formulated as constraint satisfaction
 *  problems.
 *
 *  Copyright (C) 1999  Peter van Beek and Xinguang Chen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.  See the GNU General Public License
 *  for more details (see the file called Copying or contact the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA).
 */

#include "mystery.h"
#include "../Share/global.h"


static I_type *L;
static int  squirt_available;
static void shortest_path();


void
get_instance( problem, squirt )
    int         problem, squirt;
{
    int         i, j;
    int         cargo, vehicle, nodeA, nodeB, number;
    char        name[30];
    FILE        *fp;

    if( problem < 10 ) sprintf( name, "./Problems/p0%1d", problem );
    else               sprintf( name, "./Problems/p%2d",  problem );

    if( (fp = fopen( name, "r" )) == NULL ) {
        printf( "Could not open file %s\n", name );
        exit( 0 );
    }

    squirt_available = squirt;

    L = (I_type *)new_calloc( 1, sizeof(I_type) );

    /*
     *  Get and set parameters.
     */
    L->problem = problem;
    fscanf( fp, "Parameters:\n" );
    fscanf( fp, "\tname: %s\n", &name[0] );
    fscanf( fp, "\tnodes: %d\n", &(L->nodes) );
    fscanf( fp, "\tvehicles: %d\n", &(L->vehicles) );
    fscanf( fp, "\tcargo: %d\n", &(L->cargo) );
    fscanf( fp, "\n" );

    L->path = (int **)new_calloc( L->nodes+1, sizeof(int) );
    for( i = 1; i <= L->nodes; i++ ) {
        L->path[i] = new_calloc( L->nodes+1, sizeof(int) );
    }
    for( i = 1; i <= L->nodes; i++ ) {
        L->path[i][i] = 0;
        for( j = 1; j <= L->nodes; j++ ) if( i != j ) {
            L->path[i][j] = HUGE;
        }
    }
    fscanf( fp, "Graph:\n" );
    while( fscanf( fp, "\t(adjacent node%d node%d)\n", &nodeA, &nodeB ) > 0 ) {
        L->path[nodeA][nodeB] = 1;
    }

    /*
     *  Allocate space to store the initial and goal states.
     */
    L->init_cargo    = new_calloc( L->cargo+1, sizeof(int) );
    L->goal_cargo    = new_calloc( L->cargo+1, sizeof(int) );
    L->init_vehicle  = new_calloc( L->vehicles+1, sizeof(int) );
    L->max_capacity  = new_calloc( L->vehicles+1, sizeof(int) );
    L->max_fuel      = new_calloc( L->nodes+1, sizeof(int) );

    fscanf( fp, "Initial:\n" );
    for( i = 1; i <= L->cargo; i++ ) {
        fscanf( fp, "\t(at cargo%d node%d)\n", &cargo, &nodeA );
        L->init_cargo[cargo] = nodeA;
    }
    for( i = 1; i <= L->vehicles; i++ ) {
        fscanf( fp, "\t(at vehicle%d node%d)\n", &vehicle, &nodeA );
        L->init_vehicle[vehicle] = nodeA;
    }
    for( i = 1; i <= L->vehicles; i++ ) {
        fscanf( fp, "\t(capacity vehicle%d %d)\n", &vehicle, &number );
        L->max_capacity[vehicle] = number;
    }
    for( i = 1; i <= L->nodes; i++ ) {
        fscanf( fp, "\t(fuel node%d %d)\n", &nodeA, &number );
        L->max_fuel[nodeA] = number;
        if( !squirt_action() && (number == 0) ) {
            for( j = 1; j <= L->nodes; j++ ) if (j != nodeA) {
                L->path[nodeA][j] = HUGE;
            }
        }
    }
    fscanf( fp, "\n" );

    for( i = 1; i <= L->cargo; i++ ) {
        L->goal_cargo[i] = L->init_cargo[i];
    }

    fscanf( fp, "Goal:\n" );
    while( fscanf( fp, "\t(at cargo%d node%d)\n", &cargo, &nodeA ) > 0 ) {
        L->goal_cargo[cargo] = nodeA;
    }

    shortest_path();

    L->cargo_to_move = new_calloc( L->cargo+1, sizeof(int) );
    L->n_cargo_to_move = 0;
    for( i = 1; i <= L->cargo; i++ ) {
        if( L->goal_cargo[i] != L->init_cargo[i] ) {
            L->n_cargo_to_move++;
            L->cargo_to_move[L->n_cargo_to_move] = i;
        }
    }

    fclose( fp );
}

void
print_instance()
{
    printf( "Parameters:\n" );
    printf("\tproblem: %d\n", L->problem );
    printf("\tnodes: %d\n", L->nodes );
    printf("\tvehicles: %d\n", L->vehicles );
    printf("\tcargo: %d\n", L->cargo );
}

void
dump_instance()
{
    int         i, j;

    printf( "\nGraph:\n" );
    for( i = 1; i <= L->nodes; i++ )
    for( j = 1; j <= L->nodes; j++ ) {
        if( L->path[i][j] == 1 )
            printf( "\t(adjacent node%d node%d)\n", i, j );
    }

    printf( "\nShortest paths:\n" );
    for( i = 1; i <= L->nodes; i++ ) {
        printf( "%2d: ", i );
        for( j = 1; j <= L->nodes; j++ ) {
            if( L->path[i][j] == HUGE )
                printf( "** " );
            else
                printf( "%2d ", L->path[i][j] );
        }
        printf( "\n" );
    }

    printf( "\nInitial:\n" );
    for( i = 1; i <= L->cargo; i++ ) {
        printf( "\t(at cargo%d node%d)\n", i, L->init_cargo[i] );
    }
    for( i = 1; i <= L->vehicles; i++ ) {
        printf( "\t(at vehicle%d node%d)\n", i, L->init_vehicle[i] );
    }
    for( i = 1; i <= L->vehicles; i++ ) {
        printf( "\t(capacity vehicle%d %d)\n", i, L->max_capacity[i] );
    }
    for( i = 1; i <= L->nodes; i++ ) {
        printf( "\t(fuel node%d %d)\n", i, L->max_fuel[i] );
    }

    printf( "\nGoal:\n" );
    for( i = 1; i <= L->cargo; i++ ) {
        if( L->goal_cargo[i] != L->init_cargo[i] ) {
            printf( "\t(at cargo%d node%d)\n", i, L->goal_cargo[i] );
        }
    }
}

int get_num_cargo()     { return( L->n_cargo_to_move ); } 
int get_num_vehicles()  { return( L->vehicles ); }
int get_num_nodes()     { return( L->nodes ); } 
int get_upper_bound()   { return( L->n_cargo_to_move * (2*L->nodes + 2) ); }
int squirt_action()     { return( squirt_available ); }

int get_cargo_name(   int i ) { return( L->cargo_to_move[i] ); }
int get_vehicle_name( int i ) { return( i ); }
int get_node_name(    int i ) { return( i ); }

int get_init_cargo(   int i ) { return( L->init_cargo[i] );   } 
int get_goal_cargo(   int i ) { return( L->goal_cargo[i] );   }
int get_init_vehicle( int i ) { return( L->init_vehicle[i] ); }
int get_max_capacity( int i ) { return( L->max_capacity[i] ); }
int get_max_fuel(     int i ) { return( L->max_fuel[i] );     }

static int
min( x, y )
    int x, y;
{
    if( x < y ) return( x );
    else        return( y );
}

static int
max( x, y )
    int x, y;
{
    if( x > y ) return( x );
    else        return( y );
}

static void
shortest_path()
{
    int i, j, k;

    for( k = 1; k <= L->nodes; k++ ) {
        for( i = 1; i <= L->nodes; i++ )
        for( j = 1; j <= L->nodes; j++ ) {
            L->path[i][j] = min( L->path[i][j],
                                 L->path[i][k] + L->path[k][j] );
        }
    }
}

int
sp_node_distance( nodeA, nodeB )
    int nodeA, nodeB;
{
    return( L->path[nodeA][nodeB] );
}

int
get_lower_bound()
{
    int i, j, lower_bound, min_vehicle_distance;
    int nodeA, nodeB, nodeC;

    lower_bound = 0;
    for( i = 1; i <= L->cargo; i++ ) {
        if( L->goal_cargo[i] != L->init_cargo[i] ) {
            nodeB = L->init_cargo[i];
            nodeC = L->goal_cargo[i];
            min_vehicle_distance = HUGE;
            for( j = 1; j <= L->vehicles; j++ ) {
                nodeA = L->init_vehicle[j];
                min_vehicle_distance = min( min_vehicle_distance,
                                            L->path[nodeA][nodeB] );
            }
            lower_bound = max( lower_bound,
                               min_vehicle_distance +
                               2 +
                               L->path[nodeB][nodeC] );
        }
    }

    return( lower_bound );
}
