
/*
 *  CPlan solves planning problems formulated as constraint satisfaction
 *  problems.
 *
 *  Copyright (C) 1999  Peter van Beek and Xinguang Chen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.  See the GNU General Public License
 *  for more details (see the file called Copying or contact the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA).
 */

/*
 *  Variable types for mystery planning problems.
 */
#define  C_LOC     0    /* Cargo location       */
#define  V_LOC     1    /* Vehicle location     */
#define  N_FUEL    2    /* Node fuel            */
#define  SQUIRT    3    /* MPRIME problems only */

/*
 *  Value types for mystery planning problems.
 */
#define  NODE      0
#define  VEHICLE   1
#define  NUMBER    2

/*
 *  Types of constraints for mystery planning problems.
 */
#define  DISTANCE_C_LOC_CONSTRAINT          0
#define  DISTANCE_V_LOC_CONSTRAINT          1
#define  DISTANCE_N_FUEL_CONSTRAINT         2
#define  ACTION_LOAD_UNLOAD_CONSTRAINT      3
#define  ACTION_MOVE_CONSTRAINT             4
#define  VEHICLE_CAPACITY_CONSTRAINT        5
#define  SINGLE_MOVE_CONSTRAINT             6
#define  ACTION_SQUIRT_CONSTRAINT           7  /* MPRIME problems only */
#define  SINGLE_SQUIRT_MOVE_CONSTRAINT      8  /* MPRIME problems only */
#define  LAST                               9

/*
 *  Struct for an instance of the mystery problems.
 */
typedef struct {
    int  problem;
    int  nodes;                 /* number of nodes                     */
    int  vehicles;              /* number of vehicles                  */
    int  cargo;                 /* pieces of cargo                     */
    int  **path;                /* (nodes x nodes) graph structure     */
    int  *init_cargo;           /* initial location of each cargo      */
    int  *goal_cargo;           /* goal location of each cargo         */
    int  n_cargo_to_move;       /* number of cargo that need to move   */
    int  *cargo_to_move;        /* list of cargo which need to move    */
    int  *init_vehicle;         /* initial location of each vehicle    */
    int  *max_capacity;         /* initial capacity of each vehicle    */
    int  *max_fuel;             /* initial amount of fuel at each node */
} I_type;

/*
 *  Routines available for an instance of the mystery problems.
 */
void    get_instance();
void    print_instance();
void    dump_instance();

int     get_num_cargo();
int     get_num_vehicles();
int     get_num_nodes();
int     get_lower_bound();
int     get_upper_bound();
int     squirt_action();

int     get_cargo_name();
int     get_vehicle_name();
int     get_node_name();

int     get_init_cargo();
int     get_goal_cargo();
int     get_init_vehicle();
int     get_max_capacity();
int     get_max_fuel();
int     sp_node_distance();

