
/*
 *  CPlan solves planning problems formulated as constraint satisfaction
 *  problems.
 *
 *  Copyright (C) 1999  Peter van Beek and Xinguang Chen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.  See the GNU General Public License
 *  for more details (see the file called Copying or contact the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA).
 */

#include "mystery.h"
#include "../Share/csp.h"
#include "../Share/global.h"

static int
same_attributes( a, b )
    VALA	a, b;
{
    return( a.type   == b.type &&
	    a.number == b.number );
}

void
process_solution( C, solution )
    CSP_type    *C;
    int		*solution;
{
    int		t, i;
    int		x, y;
    VALA	a, b;

    for( t = 0; t <= C->time-1; t++ ) {
	printf( "Actions at time %d\n", t+1 );
	/*
	 * Cargo variables
	 */
        for( i = 1; i <= get_num_cargo(); i++ ) {
	    x = get_var( C, t,   C_LOC, i );
	    y = get_var( C, t+1, C_LOC, i );
	    a = C->value_attribute[x][solution[x]];
	    b = C->value_attribute[y][solution[y]];

	    if( !same_attributes( a, b ) ) {
		if( a.type == NODE &&
		    b.type == VEHICLE )
		    printf( "\tload cargo%d from node%d onto vehicle%d\n",
			    C->variable_attribute[x].number,
			    a.number, b.number );
		else
		if( a.type == VEHICLE &&
		    b.type == NODE )
		    printf( "\tunload cargo%d from vehicle%d to node%d\n",
			    C->variable_attribute[x].number,
			    a.number, b.number );
		else
		    printf( "error\n" );
	    }
        }
	/*
	 * Vehicle variables
	 */
	for( i = 1; i <= get_num_vehicles(); i++ ) {
	    x = get_var( C, t,   V_LOC, i );
	    y = get_var( C, t+1, V_LOC, i );
	    a = C->value_attribute[x][solution[x]];
	    b = C->value_attribute[y][solution[y]];

	    if( !same_attributes( a, b ) ) {
		printf( "\tmove vehicle%d from node%d to node%d\n",
			C->variable_attribute[x].number,
			a.number, b.number );
	    }
        }
	/*
	 * Squirt variables
	 */
	if( squirt_action() ) {
	    x = get_var( C, t,   SQUIRT, 1 );
	    y = get_var( C, t+1, SQUIRT, 2 );
	    a = C->value_attribute[x][solution[x]];
	    b = C->value_attribute[y][solution[y]];

            if( !same_attributes( a, b ) ) {
                printf( "\tsquirt a unit of fuel from node%d to node%d\n",
                        a.number, b.number );
            }
	}
    }
}

